"{ Package: 'stx:libcomp' }"

ByteArray variableByteSubclass:#ImmutableByteArray
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'System-Compiler-Support'
!


!ImmutableByteArray methodsFor:'accessing'!

at:index put:value
    "Trigger an error if an immutable bytearray is stored into.
     The store will be performed (for compatibility reasons) if you continue
     in the debugger."

    self notifyStoreError.
    ^ super at:index put:value
!

basicAt:index put:value
    "Trigger an error if an immutable bytearray is stored into.
     The store will be performed (for compatibility reasons) if you continue
     in the debugger."

    self notifyStoreError.
    ^ super basicAt:index put:value
! !

!ImmutableByteArray methodsFor:'converting'!

asImmutableByteArray
    ^ self
! !

!ImmutableByteArray methodsFor:'copying'!

copyEmpty
    "when copying, return a real (mutable) ByteArray"

    ^ ByteArray new:self size
!

copyEmptyAndGrow:size
    "when copying, return a real (mutable) ByteArray"

    ^ ByteArray new:size
!

postCopy
    "when copied, make me a real (mutable) ByteArray"

    self changeClassTo:ByteArray.
!

postDeepCopy
    "when copied, make me a real (mutable) ByteArray"

    self changeClassTo:ByteArray.

    "
     #[1 2 3 4] asImmutableByteArray copy          class
     #[1 2 3 4] asImmutableByteArray shallowCopy   class
     #[1 2 3 4] asImmutableByteArray deepCopy      class
    "
!

shallowCopy
    "when copying, return a real (mutable) ByteArray"

    |sz|

    sz := self size.
    ^ (ByteArray new:sz)
        replaceFrom:1 to:sz with:self startingAt:1
! !

!ImmutableByteArray methodsFor:'error handling'!

creator 
    "find the method that contains me"

    ^ Method allSubInstances detect:[:aMethod | (aMethod referencesGlobal:self)] ifNone:nil.

    " 
      #(1 2 3) creator
    "

    "Modified: 24.6.1996 / 15:36:28 / stefan"
!

notifyStoreError
    "a store is attempted - for our convenience, find the method that
     contains me, for a nicer error message"

    |creator msg|

    creator := self creator.
    msg := 'store into/change of literal'.
    creator notNil ifTrue:[
        msg := msg , ' (' , creator whoString , ')'
    ].
    "
     this error is reported on an attempt to store into a literal
     array. The literal was created in creator.
     If you press continue in the debugger, the store will be performed.
     If you don't want this, press abort and check your code.
     Storing into literals is VERY VERY bad coding style.
    "
    self error:msg mayProceed:true
! !

!ImmutableByteArray methodsFor:'private'!

species
    "Copies should be mutable"

    ^ ByteArray
! !

!ImmutableByteArray methodsFor:'queries'!

isLiteral
    "return true, if the receiver can be used as a literal constant in ST syntax
     (i.e. can be used in constant arrays)"

    "yes, I must be"
    ^ true

! !

!ImmutableByteArray methodsFor:'specials'!

become:anotherObject
    "trigger an error if I should become something else
     (this would be an even more tricky manipulation)"

    self notifyStoreError.
    ^ super become:anotherObject
!

becomeNil
    "trigger an error if I should become nil
     (this would be an even more tricky manipulation)"

    self notifyStoreError.
    ^ super becomeNil
! !

!ImmutableByteArray class methodsFor:'documentation'!

version_CVS
    ^ '$Header: /cvs/stx/stx/libbasic/ImmutableByteArray.st,v 1.1 2009-11-05 14:34:35 cg Exp $'
! !
